import subprocess
import itertools

from pathlib import Path
from PIL import Image
from reportlab.pdfgen import canvas
from reportlab.lib.pagesizes import A4
from reportlab.lib.units import mm


class Logo:
    def __init__(self, path, target_height, adjust_x=0, adjust_y=0):
        self.path = path
        self.height = target_height
        self.width = self._scale_width(target_height)
        self.adjust_x = adjust_x
        self.adjust_y = adjust_y
        self.size = {"width": self.width, "height": self.height}

    def _scale_width(self, height):
        img = Image.open(self.path)
        img_w2h = img.width / img.height
        return height * img_w2h


_tmp_pos = [
    # upper card
    39 + 114,
    # lower card
    39,
]
y_positions = itertools.cycle([rp * mm for rp in _tmp_pos])
y_height = 50 * mm
y_padding = 5 * mm

x_pos = 55 * mm
x_width = 100 * mm
x_padding = 10 * mm

false_and_true = itertools.cycle([False, True])

imtek_logo_path = Path(__file__).parent / "logo_imtek.png"


class NamePlateCards:
    def __init__(
        self,
        partner_logo=None,
        partner_color=None,
        name_size=24,
        company_size=12,
        debug=False,
    ):
        self.imtek_logo = Logo(imtek_logo_path, 10 * mm)
        self.imtek_color = self._color_from_rgb(23, 17, 117)
        self.partner_logo = partner_logo
        if partner_color is None:
            self.partner_color = self._guess_partner_color()
        else:
            self.partner_color = self._color_from_rgb(partner_color)
        self.name_size = name_size
        self.company_size = company_size
        self.debug = debug
        self._canvas = None

    def generate(self, attendees_file, output_file="output.pdf", show=True):
        attendees = self.parse_attendees(attendees_file)
        for page_break, entry in zip(false_and_true, attendees):
            self.draw_card(*entry)
            if page_break:
                self.canvas.showPage()
        self.canvas.save()
        if show:
            subprocess.run(["open", output_file])

    @property
    def canvas(self):
        if self._canvas is None:
            self._canvas = canvas.Canvas("output.pdf", pagesize=A4)
        return self._canvas

    def _color_from_rgb(self, *args):
        return tuple(x / 255 for x in args)

    def _guess_partner_color(self):
        if self.partner_logo is None:
            return (0, 0, 0)
        img = Image.open(self.partner_logo.path)
        colors = sorted(img.getcolors(img.width * img.height), reverse=True)
        if colors is None:
            return (0, 0, 0)
        else:
            return self._color_from_rgb(*colors[0][1])

    def parse_attendees(self, attendees_file):
        with open(attendees_file, "r") as fh:
            lines = [l.strip() for l in fh]
        splited = [line.split("\t") for line in lines if line]
        return [(c.strip(), f.strip(), l.strip()) for c, f, l in splited]

    def box_helper(self):
        if self.debug:
            self.canvas.setStrokeColorRGB(1, 0, 0)
            self.canvas.rect(0, 0, x_width, y_height)

    def draw_card(self, company, first_name, last_name):
        self.canvas.saveState()
        y_pos = next(y_positions)
        self.canvas.translate(x_pos, y_pos)
        self.box_helper()
        self.draw_field(company, first_name, last_name)
        # next translate is relative to the first
        self.canvas.translate(x_width, y_height * 2)
        self.canvas.rotate(180)
        self.box_helper()
        self.draw_field(company, first_name, last_name)
        self.canvas.restoreState()

    def draw_field(self, company, first_name, last_name):
        self.draw_logo(self.imtek_logo, "left")
        if self.partner_logo is not None:
            self.draw_logo(self.partner_logo, "right")
        self.canvas.setFillColorRGB(*self.imtek_color)
        self.canvas.setFont("Helvetica-Bold", self.name_size)
        self.canvas.drawString(x_padding, y_padding + 18 * mm, first_name)
        self.canvas.drawString(x_padding, y_padding + 8 * mm, last_name)
        self.canvas.setFillColorRGB(*self.partner_color)
        self.canvas.setFont("Helvetica-Bold", self.company_size)
        self.canvas.drawString(x_padding, y_padding, company)

    def draw_logo(self, img, position="left"):
        if position == "left":
            x_left = x_padding
        else:
            x_left = x_width - x_padding - img.width
        self.canvas.drawImage(
            img.path,
            x_left + img.adjust_x,
            y_height - y_padding - img.height + img.adjust_y,
            **img.size,
            mask="auto"
        )
